#!/usr/bin/env python
"""
_RuntimeExpress_

Runtime Script for unpacking an express job configuration

"""

import os
import pickle

from ProdCommon.FwkJobRep.TaskState import TaskState
from ProdCommon.MCPayloads.JobSpec import JobSpec
from ProdCommon.MCPayloads.WorkflowSpec import WorkflowSpec


class NodeFinder:
    def __init__(self, nodeName):
        self.nodeName = nodeName
        self.result = None

    def __call__(self, nodeInstance):
        if nodeInstance.name == self.nodeName:
            self.result = nodeInstance

def sortByValue(d):
    """ Returns the keys of dictionary d sorted by their values """
    items = d.items()
    backitems = [ [v[1], v[0]] for v in items]
    backitems.sort()
    return [ backitems[i][1] for i in range(0, len(backitems))]


class JobSetup:
    """
    _JobSetup_

    Object to manipulate the Configuration files for an express job

    - Extract the details of the job entity stored in the config

    """
    def __init__(self, workflowSpec, jobSpec):

        self.workflowSpec = WorkflowSpec()
        self.workflowSpec.load(workflowSpec)

        self.jobSpec = JobSpec()
        self.jobSpec.load(jobSpec)

        self.cmssw = {}
        self.cmssw['version1'] = 0
        self.cmssw['version2'] = 0
        self.cmssw['version3'] = 0
        version = self.workflowSpec.parameters.get("CMSSWVersion")
        cmsswVersionList = version.split("_")
        try:
            self.cmssw['version1'] = int(cmsswVersionList[1])
            self.cmssw['version2'] = int(cmsswVersionList[2])
            self.cmssw['version3'] = int(cmsswVersionList[3])
        except ValueError:
            print "Warning: non standard CMSSW version: %s" % version

        taskState = TaskState(os.getcwd())
        taskState.loadRunResDB()

        jobSpecFinder = NodeFinder(taskState.taskName())
        self.jobSpec.payload.operate(jobSpecFinder)
        self.jobSpecNode = jobSpecFinder.result

##         workflowFinder = NodeFinder(taskState.taskName())
##         self.workflowSpec.payload.operate(workflowFinder)
##         self.workflowNode = workflowFinder.result

        self.run = None
        self.lumis = []
        self.streamerFiles = []


    def unpackJobEntity(self):
        """
        _unpackJobEntity_

        Get the StreamerJobEntity from the JobSpec node

        """
        jobEntity = self.jobSpecNode.cfgInterface.extensions.get("Streamer", None)
        if jobEntity == None:
            msg = "No StreamerJobEntity in JobSpec configuration\n"
            msg += "This is required for express jobs\n"
            raise RuntimeError, msg

        # Get run and lumi numbers for this job
        self.run = jobEntity.data['runNumber']
        self.lumis = jobEntity.data['lumiSections']
        print "Express Job Handling Run:%s\n LumiSections: %s\n" % (self.run,self.lumis)

        # Sort streamer input by lumi ID for time ordering
        self.streamerFiles = sortByValue(jobEntity.data['streamerFiles'])
        msg = "Streamer Files for this job are:\n"
        for streamer in self.streamerFiles:
            msg += "  %s\n" % streamer
        print msg

        return


    def backupPSet(self,filename,process):
        """
        _backupPSet_
        
        Write a backup copy of the current PSet to disk.
        """
        print "Wrote current configurations as %s" % filename
        handle = open(filename, 'w')
        handle.write("import pickle\n")
        handle.write("pickledCfg=\"\"\"%s\"\"\"\n" % pickle.dumps(process))
        handle.write("process = pickle.loads(pickledCfg)\n")
        handle.close()

        return


    def importAndBackupProcess(self):
        """
        _importAndBackupProcess_
        
        Try to import the process object for the job, which is contained in
        PSet.py and save a backup copy of it.
        """
        try:
            from PSet import process
        except ImportError, ex:
            msg = "Failed to import PSet module containing cmsRun Config\n"
            msg += str(ex)
            raise RuntimeError, msg

        print "PSet.py imported"

        self.backupPSet("PSetPreRuntime.log",process)

        return process


    def dumpConfiguration(self,process):
        """
        _dumpConfiguration_
        
        Write out the final configuration used for the job.
        """
        if ( ( self.cmssw['version1'] > 2 ) or \
             ( self.cmssw['version1'] == 2 and self.cmssw['version2'] >= 1 ) ):
            pycfgDump = open("PyCfgFileDump.log", 'w')
            try:
                print "Writing out PyCfgFileDump.log"
                pycfgDump.write(process.dumpPython())
            except Exception, ex:
                msg = "Error writing Python format config dump:\n"
                msg += "%s\n" % str(ex)
                msg += "This needs to be reported to the framework team"
                pycfgDump.write(msg)
            pycfgDump.close()
        else:
            cfgDump = open("CfgFileDump.log", 'w')
            try:
                print "Writing out CfgFileDump.log"
                cfgDump.write(process.dumpConfig())
            except Exception, ex:
                msg = "Error writing CfgFile format config dump\n"
                msg += "%s\n" % str(ex)
                msg += "This needs to be reported to the framework team"
                cfgDump.write(msg)
            cfgDump.close()

        return


    def importPSet(self):
        """
        _importPSet_

        Import the process object for cmsRun

        """
        process = self.importAndBackupProcess()

        # insert input files
        process.source.fileNames = self.streamerFiles
        print "Streamers inserted"

        # dump and save final configuration
        self.dumpConfiguration(process)
        self.backupPSet("PSet.py",process)

        return


def main():
    """
    _main_

    Verify that the environment variable that points at the workflow spec
    has been set correctly and then make modification to the process object

    """
    print "=========Express Job Setup================="

    workflowSpec = os.environ.get("PRODAGENT_WORKFLOW_SPEC", None)
    if workflowSpec == None:
        msg = "Unable to find WorkflowSpec from "
        msg += "PRODAGENT_WORKFLOW_SPEC variable\n"
        msg += "Unable to proceed\n"
        raise RuntimeError, msg

    if not os.path.exists(workflowSpec):
        msg = "Cannot find WorkflowSpec file:\n %s\n" % workflowSpec
        msg += "Unable to proceed\n"
        raise RuntimeError, msg

    jobSpec = os.environ.get("PRODAGENT_JOBSPEC", None)
    if jobSpec == None:
        msg = "Unable to find JobSpec from "
        msg += "PRODAGENT_JOBSPEC variable\n"
        msg += "Unable to proceed\n"
        raise RuntimeError, msg

    if not os.path.exists(jobSpec):
        msg = "Cannot find JobSpec file:\n %s\n" % jobSpec
        msg += "Unable to proceed\n"
        raise RuntimeError, msg

    instance = JobSetup(workflowSpec,jobSpec)
    instance.unpackJobEntity()
    instance.importPSet()

    print "=========Express Job Setup Done==========="

if __name__ == '__main__':
    main()
