"""
API interface specific to RepackerInjector
"""

__revision__ = "$Id: RepackerInjectorAPI.py,v 1.59 2010/07/21 09:17:10 hufnagel Exp $"
__version__ = "$Revision: 1.59 $"

import logging

from T0.DataStructs.StreamerJobEntity import StreamerJobEntity

from T0.State.Database.Reader.ListStreamers import getAvailableRepackStreamers
from T0.State.Database.Reader.ListStreamers import getAvailableConversionStreamers
from T0.State.Database.Reader.ListStreamers import getMaxRepackLumi

from T0.State.Database.Reader.ListJobs import listNewRepackJobs
from T0.State.Database.Reader.ListJobs import getNextJobID

from T0.State.Database.Writer.InsertJob import insertJobDef
from T0.State.Database.Writer.InsertJob import updateJobDef
from T0.State.Database.Writer.InsertJob import assocRepackJobToStreamer
from T0.State.Database.Writer.InsertStreamer import updateSplitStreamers


def getNewRepackJobs(dbConn):
    """
    _getNewRepackJobs_

    Retrieve all RepackerScheduler defined jobs from T0AST
    The list I get back contains duplicates (one entry per
    input file per job), so I have to process it out and then
    create and return a list of StreamerJobEntity objects

    STREAM_ID and RUN_ID are identical for each input file
    for a given job (enforced in scheduling rules)

    """
    jobs = listNewRepackJobs(dbConn)

    jobEntityDict = {}
    for job in jobs:
        jobEntity = jobEntityDict.get(job['JOB_ID'], None)
        if jobEntity == None:
            jobEntity = StreamerJobEntity(
                jobID = job['JOB_ID'],
                streamerFiles = { job['LFN'] : job['LUMI_ID'] },
                runNumber = job['RUN_ID'],
                lumiSections = [ job['LUMI_ID'] ],
                streamID = job['STREAM_ID']
                )
            jobEntityDict[job['JOB_ID']] = jobEntity
        else:
            jobEntity['streamerFiles'][job['LFN']] = job['LUMI_ID']
            jobEntity['lumiSections'].append(job['LUMI_ID'])
            
        # remove duplicates
        jobEntity['lumiSections'] = list(set(jobEntity['lumiSections']))

    return sortJobEntitiesByRunStream(jobEntityDict.values())


def defineConversionJobs(dbConn):
    """
    _defineConversionJobs_

    builds a new conversion job per streamer file

    """
    streamers = getAvailableConversionStreamers(dbConn)

    jobEntityList = []
    for streamer in streamers:
        jobEntity = StreamerJobEntity(
            jobID = getNextJobID(dbConn, "Repack"),
            streamerFiles = { streamer['LFN'] : streamer['LUMI_ID'] },
            runNumber = streamer['RUN_ID'],
            lumiSections = [ streamer['LUMI_ID'] ],
            streamerIDs = [ streamer['STREAMER_ID'] ],
            streamID = streamer['STREAM_ID'],
            )
        jobEntityList.append(jobEntity)

    return sortJobEntitiesByRunStream(jobEntityList)


def defineRepackJobs(dbConn,
                     maxStreamerSizeSingleLumi,
                     maxStreamerEventsSingleLumi,
                     maxStreamerSizeMultiLumi,
                     maxStreamerEventsMultiLumi,
                     maxStreamerCount):
    """
    _defineJobs_

    builds new repack jobs per run, stream and lumi
    supports putting more than one ls per repack job

    """
    streamers = getAvailableRepackStreamers(dbConn)


    streamerDict = {}
    closeoutDict = {}
    for streamer in streamers:

        run = streamer['RUN_ID']
        stream = streamer['STREAM_ID']
        lumi = streamer['LUMI_ID']

        if streamerDict.has_key(run):
            if streamerDict[run].has_key(stream):
                if streamerDict[run][stream].has_key(lumi):
                    streamerDict[run][stream][lumi].append(streamer)
                else:
                    streamerDict[run][stream][lumi] = [ streamer ]
            else:
                streamerDict[run][stream] = { lumi : [ streamer ] }
        else:
            streamerDict[run] = { stream : { lumi : [ streamer ] } }

        if not closeoutDict.has_key(run):
            closeoutDict[run] = streamer['CLOSEOUT']


    jobEntityList = []
    for run in sorted(streamerDict.keys()):

        for stream in sorted(streamerDict[run].keys()):

            # schedule repack jobs based on size and event limits

            jobSizeTotal = 0
            jobEventsTotal = 0
            jobStreamerList = []

            for lumi in sorted(streamerDict[run][stream].keys()):

                lumiStreamerList = streamerDict[run][stream][lumi]

                # calculate new total size and event
                lumiSizeTotal = 0
                lumiEventsTotal = 0
                for streamer in lumiStreamerList:
                    lumiSizeTotal += streamer['FILESIZE']
                    lumiEventsTotal += streamer['EVENTS']
                        
                # lumi is larger than split limits
                #
                # => handle lumi individually and split
                #
                if lumiSizeTotal > maxStreamerSizeSingleLumi or \
                       lumiEventsTotal > maxStreamerEventsSingleLumi:

                    while len(lumiStreamerList) > 0:

                        eventsTotal = 0
                        sizeTotal = 0
                        streamerList = []
                        for streamer in lumiStreamerList:

                            # if first streamer, always use it
                            if len(streamerList) == 0:
                                eventsTotal = streamer['EVENTS']
                                sizeTotal = streamer['FILESIZE']
                                streamerList.append(streamer)
                            # otherwise calculate new totals and check if to use streamer
                            else:
                                newEventsTotal = eventsTotal + streamer['EVENTS']
                                newSizeTotal = sizeTotal + streamer['FILESIZE']                        

                                if newSizeTotal <= maxStreamerSizeSingleLumi and \
                                       newEventsTotal <= maxStreamerEventsSingleLumi:
                                    eventsTotal = newEventsTotal
                                    sizeTotal = newSizeTotal
                                    streamerList.append(streamer)

                        jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                                       runNumber = run,
                                                       streamID = stream,
                                                       forceUnmergedOutput = True )
                        jobEntity.addStreamers(streamerList)
                        jobEntityList.append(jobEntity)

                        for streamer in streamerList:
                            lumiStreamerList.remove(streamer)

                # lumi is smaller than split limits
                # check if it can be combined with previous lumi(s)
                #
                # yes => just add lumi to job (with an additional order check)
                #
                # no => issue job for previous lumi(s), save current for next job
                #
                else:

                    newSizeTotal = jobSizeTotal + lumiSizeTotal
                    newEventsTotal = jobEventsTotal + lumiEventsTotal
                    newStreamerCount = len(jobStreamerList) + len(lumiStreamerList)

                    # always take the first one
                    if len(jobStreamerList) == 0:

                        jobSizeTotal = newSizeTotal
                        jobEventsTotal = newEventsTotal
                        jobStreamerList.extend(lumiStreamerList)
                        lastLumi = lumi

                    elif newSizeTotal <= maxStreamerSizeMultiLumi and \
                             newEventsTotal <= maxStreamerEventsMultiLumi and \
                             newStreamerCount <= maxStreamerCount:

                        jobSizeTotal = newSizeTotal
                        jobEventsTotal = newEventsTotal
                        jobStreamerList.extend(lumiStreamerList)
                        lastLumi = lumi

                    else:

                        jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                                       runNumber = run,
                                                       streamID = stream )

                        jobEntity.addStreamers(jobStreamerList)
                        jobEntityList.append(jobEntity)

                        jobSizeTotal = lumiSizeTotal
                        jobEventsTotal = lumiEventsTotal
                        jobStreamerList = lumiStreamerList
                        lastLumi = lumi

            # if we are in closeout issue repack job for leftovers
            if len(jobStreamerList) > 0 and closeoutDict[run]:
                jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                               runNumber = run,
                                               streamID = stream )
                jobEntity.addStreamers(jobStreamerList)
                jobEntityList.append(jobEntity)

    return sortJobEntitiesByRunStream(jobEntityList)


def defineStrictRepackJobs(dbConn,
                           maxStreamerSizeSingleLumi,
                           maxStreamerEventsSingleLumi,
                           maxStreamerSizeMultiLumi,
                           maxStreamerEventsMultiLumi,
                           maxStreamerCount):
    """
    _defineStrictJobs_

    builds new repack jobs per run, stream and lumi
    supports putting more than one ls per repack job

    """
    streamers = getAvailableRepackStreamers(dbConn)


    streamerDict = {}
    closeoutDict = {}
    for streamer in streamers:

        run = streamer['RUN_ID']
        stream = streamer['STREAM_ID']
        lumi = streamer['LUMI_ID']

        if streamerDict.has_key(run):
            if streamerDict[run].has_key(stream):
                if streamerDict[run][stream].has_key(lumi):
                    streamerDict[run][stream][lumi].append(streamer)
                else:
                    streamerDict[run][stream][lumi] = [ streamer ]
            else:
                streamerDict[run][stream] = { lumi : [ streamer ] }
        else:
            streamerDict[run] = { stream : { lumi : [ streamer ] } }

        if not closeoutDict.has_key(run):
            closeoutDict[run] = streamer['CLOSEOUT']


    jobEntityList = []
    for run in sorted(streamerDict.keys()):

        for stream in sorted(streamerDict[run].keys()):

            # schedule repack jobs based on size and event limits

            jobSizeTotal = 0
            jobEventsTotal = 0
            jobStreamerList = []

            # get largest lumi with repack jobs for run/stream
            lastLumi = getMaxRepackLumi(dbConn, run, stream)

            for lumi in sorted(streamerDict[run][stream].keys()):

                lumiStreamerList = streamerDict[run][stream][lumi]

                # calculate lumi size and event count
                lumiSizeTotal = 0
                lumiEventsTotal = 0
                for streamer in lumiStreamerList:
                    lumiSizeTotal += streamer['FILESIZE']
                    lumiEventsTotal += streamer['EVENTS']

                # if we miss a lumi => bail unless we are in forceMergeCloseout
                if lumi != lastLumi + 1:
                    if closeoutDict[run]:
                        if len(jobStreamerList) > 0:
                            jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                                           runNumber = run,
                                                           streamID = stream )
                            jobEntity.addStreamers(jobStreamerList)
                            jobEntityList.append(jobEntity)

                            jobSizeTotal = 0
                            jobEventsTotal = 0
                            jobStreamerList = []
                    else:
                        break

                lastLumi = lumi

                # lumi is larger than split limits
                #
                # => handle lumi individually and split
                #
                if lumiSizeTotal > maxStreamerSizeSingleLumi or \
                       lumiEventsTotal > maxStreamerEventsSingleLumi:

                    # trigger merge on what we have to preserve order
                    if len(jobStreamerList) > 0:
                        jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                                       runNumber = run,
                                                       streamID = stream )
                        jobEntity.addStreamers(jobStreamerList)
                        jobEntityList.append(jobEntity)

                        jobSizeTotal = 0
                        jobEventsTotal = 0
                        jobStreamerList = []

                    while len(lumiStreamerList) > 0:

                        eventsTotal = 0
                        sizeTotal = 0
                        streamerList = []
                        for streamer in lumiStreamerList:

                            # if first streamer, always use it
                            if len(streamerList) == 0:
                                eventsTotal = streamer['EVENTS']
                                sizeTotal = streamer['FILESIZE']
                                streamerList.append(streamer)
                            # otherwise calculate new totals and check if to use streamer
                            else:
                                newEventsTotal = eventsTotal + streamer['EVENTS']
                                newSizeTotal = sizeTotal + streamer['FILESIZE']                        

                                if newSizeTotal <= maxStreamerSizeSingleLumi and \
                                       newEventsTotal <= maxStreamerEventsSingleLumi:
                                    eventsTotal = newEventsTotal
                                    sizeTotal = newSizeTotal
                                    streamerList.append(streamer)

                        jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                                       runNumber = run,
                                                       streamID = stream,
                                                       forceUnmergedOutput = True )
                        jobEntity.addStreamers(streamerList)
                        jobEntityList.append(jobEntity)

                        for streamer in streamerList:
                            lumiStreamerList.remove(streamer)

                # lumi is smaller than split limits
                # check if it can be combined with previous lumi(s)
                #
                # yes => just add lumi to job (with an additional order check)
                #
                # no => issue job for previous lumi(s), save current for next job
                #
                else:

                    newSizeTotal = jobSizeTotal + lumiSizeTotal
                    newEventsTotal = jobEventsTotal + lumiEventsTotal
                    newStreamerCount = len(jobStreamerList) + len(lumiStreamerList)

                    # still safe with new lumi, just add it
                    if newSizeTotal <= maxStreamerSizeMultiLumi and \
                           newEventsTotal <= maxStreamerEventsMultiLumi and \
                           newStreamerCount <= maxStreamerCount:

                        jobSizeTotal = newSizeTotal
                        jobEventsTotal = newEventsTotal
                        jobStreamerList.extend(lumiStreamerList)

                    # over limits with new lumi, issue repack job
                    else:

                        jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                                       runNumber = run,
                                                       streamID = stream )
                        jobEntity.addStreamers(jobStreamerList)
                        jobEntityList.append(jobEntity)

                        jobSizeTotal = lumiSizeTotal
                        jobEventsTotal = lumiEventsTotal
                        jobStreamerList = lumiStreamerList

            # if we are in closeout issue repack job for leftovers
            if len(jobStreamerList) > 0 and closeoutDict[run]:
                jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Repack"),
                                               runNumber = run,
                                               streamID = stream )
                jobEntity.addStreamers(jobStreamerList)
                jobEntityList.append(jobEntity)

    return sortJobEntitiesByRunStream(jobEntityList)


def sortJobEntitiesByRunStream(jobEntityList):
    """
    _sortJobEntitiesByRunStream_

    take a list of JobEntity and turns it into a
    nested dictionary, separated by run and stream

    """
    jobEntityDict = {}
    for jobEntity in jobEntityList:

        run = jobEntity["runNumber"]
        stream = jobEntity["streamID"]

        if jobEntityDict.has_key(run):
            if jobEntityDict[run].has_key(stream):
                jobEntityDict[run][stream].append(jobEntity)
            else:
                jobEntityDict[run][stream] = [ jobEntity ]
        else:
            jobEntityDict[run] = { stream : [ jobEntity ] }

    return jobEntityDict


def updateJobs(dbConn, jobEntityList):
    """
    _updateJobs_

    update repack job status to Used

    """
    for jobEntity in jobEntityList:
        updateJobDef(dbConn, "Repack", jobEntity["jobID"], "Used")


def registerJobs(dbConn, jobEntityList):
    """
    _registerJobs_

    register list of repack jobs in T0AST
    
    For each job
      create a new job definition & get ID from database
      update streamer status
      insert job to streamer association

    """
    streamerIDs = []
    for jobEntity in jobEntityList:

        insertJobDef(dbConn, "Repack", "Used", jobEntity['jobID'])

        assocRepackJobToStreamer(dbConn, jobEntity['jobID'], jobEntity['streamerIDs'])

        streamerIDs.extend(jobEntity['streamerIDs'])

    updateSplitStreamers(dbConn, streamerIDs)
