#!/usr/bin/env python
"""
_ListJobs_

Utilities to retrieve information from the jobs tables in T0AST.
"""

__revision__ = "$Id: ListJobs.py,v 1.35 2010/08/03 12:03:36 hufnagel Exp $"
__version__ = "$Revision: 1.35 $"

import logging

from T0 import Globals


def getNextJobID(dbConn, jobType):
    """
    _getNextJobID_

    gets next free job id depening on job type

    """
    if jobType == "Repack":
        tableName = "repack_job_def"
    elif jobType == "Express":
        tableName = "express_job_def"
    else:
        logging.error("InsertJobDef: Unknown job type: %s" % jobType)
        return None

    idQuery = "SELECT %s_SEQ.nextval from dual" % tableName
    dbConn.execute(idQuery, useTransaction = False)
    jobID = dbConn.fetchall()

    while type(jobID) != int:
        jobID = jobID[0]

    return jobID

def listJobStatusByID(dbConn, jobType, jobID):
    """
    _listJobStatusByID_

    Return a string containing the status of a job.  This supports the
    following job types:
      Repack
      some other job type maybe need to be added (express stream)

    If the jobID is not in T0AST None is returned.
    """
    if jobType == "Repack":
        jobTable = "repack_job_def"
    elif jobType == "Express":
        jobTable = "express_job_def"
    else:
        logging.error("listJobStatusByID: Unknown Job types %s" % jobType)
        return None

    sqlQuery = """SELECT job_status.STATUS FROM %s INNER JOIN job_status ON
                  job_status.ID = %s.JOB_STATUS WHERE %s.JOB_ID = :p_1""" % \
                  (jobTable, jobTable, jobTable)

    bindVars = {"p_1": jobID}
    dbConn.execute(sqlQuery, bindVars)
    result = dbConn.fetchall()

    if len(result) == 0:
        return None

    while type(result) != str:
        result = result[0]

    return result

def listJobIDByStatus(dbConn, jobType, jobStatus):
    """
    _listJobIDByStatus_

    Return a list of jobs that have a particular status.  The jobType parameter
    must be one of the following:
      Repack
    """
    if jobType == "Repack":
        jobTable = "repack_job_def"
    elif jobType == "Express":
        jobTable = "express_job_def"
    else:
        logging.error("listJobIDByStatus: Unknown Job types %s" % jobType)
        return None

    sqlQuery = """SELECT JOB_ID FROM %s WHERE JOB_STATUS = (SELECT ID from
                  JOB_STATUS where STATUS = :p_1)""" % jobTable
                    
    bindVars = {"p_1": jobStatus}
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    jobIDs = []
    for result in results:
        jobIDs.append(result[0])

    return jobIDs


def listNewRepackJobs(dbConn):
    """
    _listNewRepackJobs_

    Special method to list all new repack jobs
    Includes lumi_id, stream_id and lfn (needed for job creation)

    Returns duplicates for jobs with multiple inputs,
    one per input files

    """
    sqlQuery = """SELECT a.job_id, c.run_id, c.lumi_id, c.stream_id, c.lfn FROM repack_job_def a
                  INNER JOIN repack_job_streamer_assoc b
                  ON a.job_id = b.job_id
                  INNER JOIN streamer c
                  ON b.streamer_id = c.streamer_id
                  INNER JOIN run_stream_style_assoc d
                  ON d.run_id = c.run_id AND d.stream_id = c.stream_id
                  INNER JOIN processing_style e
                  ON e.id = d.style_id
                  WHERE a.job_status = (SELECT id FROM job_status WHERE status = 'New')
                  AND e.name = 'Bulk'
                  GROUP BY a.job_id, c.run_id, c.lumi_id, c.stream_id, c.lfn
                  ORDER BY a.job_id"""

    dbConn.execute(sqlQuery, useTransaction = False)
    results = dbConn.fetchall()

    jobs = []
    for result in results:
        jobDict = {}
        jobDict['JOB_ID'] = result[0]
        jobDict['RUN_ID'] = result[1]
        jobDict['LUMI_ID'] = result[2]
        jobDict['STREAM_ID'] = result[3]
        jobDict['LFN'] = result[4]
        jobs.append(jobDict)

    return jobs


def listNewWMBSJobs(dbConn, subscription):
    """
    _listNewWMBSJobs_

    gets all not yet created jobs for a certain subscription
    also return additional information for injector component
    sort the files for each job by first lumi and first event

    """
    sqlQuery = """SELECT wmbs_job.id,
                         wmbs_job_mask.FirstEvent,
                         wmbs_job_mask.LastEvent,
                         wmbs_file_details.lfn,
                         wmbs_file_run_map.run,
                         wmbs_file_dataset_path_assoc.dataset_path_id
                  FROM wmbs_job
                  INNER JOIN wmbs_jobgroup ON
                    wmbs_jobgroup.id = wmbs_job.jobgroup AND
                    wmbs_jobgroup.subscription = :subscription
                  INNER JOIN wmbs_job_mask ON
                    wmbs_job_mask.job = wmbs_job.id
                  INNER JOIN wmbs_job_assoc ON
                    wmbs_job_assoc.job = wmbs_job.id
                  INNER JOIN wmbs_file_details ON
                    wmbs_file_details.id = wmbs_job_assoc.fileid
                  INNER JOIN (
                    SELECT wmbs_file_runlumi_map.fileid AS fileid,
                           wmbs_file_runlumi_map.run AS run,
                           MIN(wmbs_file_runlumi_map.lumi) AS first_lumi
                    FROM wmbs_file_runlumi_map
                    GROUP BY wmbs_file_runlumi_map.fileid,
                             wmbs_file_runlumi_map.run
                  ) wmbs_file_run_map ON
                    wmbs_file_run_map.fileid = wmbs_job_assoc.fileid
                  INNER JOIN wmbs_file_dataset_path_assoc ON
                    wmbs_file_dataset_path_assoc.file_id = wmbs_job_assoc.fileid
                  WHERE wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'new')
                  ORDER BY wmbs_file_run_map.run,
                           wmbs_file_run_map.first_lumi,
                           wmbs_file_details.first_event
                  """

    bindVars = { 'subscription' : subscription }
    dbConn.execute(sqlQuery, subscription, useTransaction = False)
    results = dbConn.fetchall()

    wmbsJobs = {}
    for result in results:
        id = result[0]
        if wmbsJobs.has_key(id):
            wmbsJob = wmbsJobs[id]
            wmbsJob['files'].append(result[3])
        else:
            wmbsJob = {}
            wmbsJob['id'] = result[0]
            wmbsJob['first_event'] = result[1]
            wmbsJob['last_event'] = result[2]
            wmbsJob['files'] = [result[3]]
            wmbsJob['run'] = result[4]
            wmbsJob['datasetpath'] = result[5]
            wmbsJobs[id] = wmbsJob

    return wmbsJobs.values()


def listNewExpressMergeJobs(dbConn, subscription):
    """
    _listNewExpressMergeJobs_

    temporary express files are not associated to datasetpath
    therefore the generic listNewWMBSJobs method does not work

    gets all not yet created jobs for a certain subscription
    also return additional information for injector component
    sort the files for each job by first lumi and first event

    no grouping needed since temporary express files
    only have a single run/lumi

    """
    sqlQuery = """SELECT wmbs_job.id,
                         wmbs_file_details.lfn,
                         wmbs_file_runlumi_map.run,
                         express_file_info.primary_dataset,
                         data_tier.name
                  FROM wmbs_job
                  INNER JOIN wmbs_jobgroup ON
                    wmbs_jobgroup.id = wmbs_job.jobgroup AND
                    wmbs_jobgroup.subscription = %d
                  INNER JOIN wmbs_job_assoc ON
                    wmbs_job_assoc.job = wmbs_job.id
                  INNER JOIN wmbs_file_details ON
                    wmbs_file_details.id = wmbs_job_assoc.fileid
                  INNER JOIN wmbs_file_runlumi_map ON
                    wmbs_file_runlumi_map.fileid = wmbs_job_assoc.fileid
                  INNER JOIN express_file_info ON
                    express_file_info.file_id = wmbs_job_assoc.fileid
                  INNER JOIN data_tier ON
                    express_file_info.data_tier = data_tier.id
                  WHERE wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'new')
                  ORDER BY wmbs_file_runlumi_map.run,
                           wmbs_file_runlumi_map.lumi,
                           wmbs_file_details.first_event
                  """ % subscription

    dbConn.execute(sqlQuery, useTransaction = False)
    results = dbConn.fetchall()

    wmbsJobs = {}
    for result in results:
        id = result[0]
        if wmbsJobs.has_key(id):
            wmbsJob = wmbsJobs[id]
            wmbsJob['files'].append(result[1])
        else:
            wmbsJob = {}
            wmbsJob['id'] = result[0]
            wmbsJob['files'] = [result[1]]
            wmbsJob['run'] = result[2]
            wmbsJob['primds_id'] = result[3]
            wmbsJob['datatier'] = result[4]
            wmbsJobs[id] = wmbsJob

    return wmbsJobs.values()


def listNewRepackJobs(dbConn, subscription):
    """
    _listNewRepackJobs_

    streamer files are not associated to run,lumi or datasetpath
    therefore the generic listNewWMBSJobs method does not work

    gets all not yet created jobs for a certain subscription
    also return additional information for injector component
    sort the files for each job by run, lumi and streamer id

    """
    sqlQuery = """SELECT wmbs_job.id,
                         streamer.run_id,
                         streamer.lumi_id,
                         streamer.stream_id,
                         streamer.lfn
                  FROM wmbs_job
                  INNER JOIN wmbs_jobgroup ON
                    wmbs_jobgroup.id = wmbs_job.jobgroup AND
                    wmbs_jobgroup.subscription = %d
                  INNER JOIN wmbs_job_assoc ON
                    wmbs_job_assoc.job = wmbs_job.id
                  INNER JOIN streamer ON
                    streamer.streamer_id = wmbs_job_assoc.fileid
                  WHERE wmbs_job.state = (SELECT id FROM wmbs_job_state WHERE name = 'new')
                  ORDER BY streamer.run_id,
                           streamer.lumi_id,
                           streamer.streamer_id
                  """ % subscription

    dbConn.execute(sqlQuery, useTransaction = False)
    results = dbConn.fetchall()

    #
    # now sort results by run and stream (optimizes workflow creation)
    #
    # needs to provide mapping of job id to input file lfn
    #
    # also needs to find lumis with multiple jobs per run/stream
    # these are repack jobs with lumi splitting and need to
    # force unmerged output
    #
    jobDict = {}
    for result in results:

        id = result[0]
        run = result[1]
        lumi = result[2]
        stream_id = result[3]
        lfn = result[4]

        if not jobDict.has_key(run):
            jobDict[run] = {}
        if not jobDict[run].has_key(stream_id):
            jobDict[run][stream_id] = { 'lumis' : {},
                                        'files' : {} }

        # convinience variable
        lumisDict = jobDict[run][stream_id]['lumis']
        filesDict = jobDict[run][stream_id]['files']

        # keep a lumi to job id reference
        # allows to check which lumis have multiple jobs
        if not lumisDict.has_key(lumi):
            lumisDict[lumi] = set([id])
        else:
            lumisDict[lumi].add(id)

        # keep a job id to lfn reference
        # needed by the RepackerInjector
        if not filesDict.has_key(id):
            filesDict[id] = [lfn]
        elif lfn not in filesDict[id]:
            filesDict[id].append(lfn)

    for run in jobDict.keys():
        for stream_id in jobDict[run].keys():

            jobDict[run][stream_id]['normal'] = {}
            jobDict[run][stream_id]['split'] = {}

            # convinience variable
            lumisDict = jobDict[run][stream_id]['lumis']
            filesDict = jobDict[run][stream_id]['files']
            normalDict = jobDict[run][stream_id]['normal']
            splitDict = jobDict[run][stream_id]['split']

            # a job is either single lumi split or multiple lumis, not both
            # based on that divide jobs into normal and split lumi jobs
            for lumi in sorted(lumisDict.keys()):
                jobIds = lumisDict[lumi]
                if len(jobIds) == 1:
                    id = jobIds.pop()
                    if not normalDict.has_key(id):
                        normalDict[id] = filesDict[id]
                else:
                    for id in jobIds:
                        splitDict[id] = filesDict[id]

            lumisDict = None
            filesDict = None
            del jobDict[run][stream_id]['lumis']
            del jobDict[run][stream_id]['files']

    return jobDict
