#!/usr/bin/env python
"""
_ListRuns_

Utilities to retrieve information from the run table in T0AST.
"""

__revision__ = "$Id: ListRuns.py,v 1.38 2010/07/19 14:31:57 hufnagel Exp $"
__version__ = "$Revision: 1.38 $"

import logging

from T0 import Globals


def listRunsByStatus(dbConn, status):
    """
    _listRunsByStatus_

    return all runs with the given status

    """
    sqlQuery = """SELECT run_id
                  FROM run
                  WHERE run_status = (SELECT id FROM run_status WHERE status = :STATUS)"""

    bindVars = { 'STATUS' : status }
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    runs = []
    for result in results:
        runs.append(result[0])

    return runs


def listRunsInCloseout(dbConn):
    """
    _listRunsInCloseout_

    get all runs being closed out, ie, whose
    status is not Active and not Complete

    also return status

    """
    sqlQuery = """SELECT run.run_id, run_status.status
                  FROM run
                  INNER JOIN run_status ON
                    run.run_status = run_status.id
                  WHERE run_status.status NOT IN ( 'Active', 'Complete' )"""

    dbConn.execute(sqlQuery, useTransaction = False)
    results = dbConn.fetchall()

    runs = {}
    for result in results:
        runs[result[0]] = result[1]

    return runs


def listRecoStartedForRun(dbConn, runNumber):
    """
    _listRecoStartedForRun_

    Determine whether or not reconstruction jobs have been released to
    the farm for a particular run.  This will return True if jobs have bene
    released, False otherwise.

    """
    sqlQuery = "SELECT reco_started FROM run WHERE run_id = :p_1"

    bindVars = {"p_1": runNumber}
    dbConn.execute(sqlQuery, bindVars)
    result = dbConn.fetchall()

    if result[0][0] == 1:
        return True

    return False


def listRunState(dbConn, runNumbers):
    """
    _listRunState_

    Retrieve the state of a run for a given run number.

    """
    if type(runNumbers) != list:
        runNumbers = [runNumbers]

    sqlQuery = """SELECT run.run_id, run_status.status
                  FROM run
                  INNER JOIN run_status ON
                    run_status.id = run.run_status
                  WHERE run.run_id = :p_1"""

    bindVars = []
    for runNumber in runNumbers:
        bindVar = {"p_1": runNumber}
        bindVars.append(bindVar)
        
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    runStatus = {}
    for result in results:
        runStatus[result[0]] = result[1]

    return runStatus


def listExpressProcCompleteForRun(dbConn, runNumber):
    """
    _listExpressProcCompleteForRun_

    Determine whether or not a run has completed express processing.
    """
    sqlQuery = """SELECT COUNT(*)
                  FROM streamer
                  INNER JOIN run_stream_style_assoc ON
                    run_stream_style_assoc.stream_id = streamer.stream_id AND
                    run_stream_style_assoc.run_id = :run AND
                    run_stream_style_assoc.style_id = (SELECT id FROM processing_style WHERE name = 'Express')
                  WHERE checkForZeroState(streamer.used) = 0
                  AND streamer.run_id = :run
                  """

    bindVars = {"run": runNumber}
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    logging.debug("Unused express streamer for run %d: %d" % (runNumber, results[0][0]))

    if results[0][0] != 0:
        return False

    sqlQuery = """SELECT COUNT(*)
                  FROM express_job_def
                  INNER JOIN express_job_streamer_assoc ON
                    express_job_streamer_assoc.job_id = express_job_def.job_id
                  INNER JOIN streamer ON
                    streamer.streamer_id = express_job_streamer_assoc.streamer_id AND
                    streamer.run_id = :run
                  WHERE express_job_def.job_status IN (
                    (SELECT id FROM job_status WHERE status = 'New'),
                    (SELECT id FROM job_status WHERE status = 'Used') )
                  """

    bindVars = {"run": runNumber}
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    logging.debug("Unfinished express jobs for run %d" % runNumber)

    if results[0][0] != 0:
        return False

    expressmergeSubscription = Globals.retrieveExpressMergeSubscription()

    sqlQuery = """SELECT COUNT(*)
                  FROM wmbs_fileset_files
                  LEFT OUTER JOIN wmbs_sub_files_complete ON
                    wmbs_sub_files_complete.fileid = wmbs_fileset_files.fileid AND
                    wmbs_sub_files_complete.subscription = %d
                  LEFT OUTER JOIN wmbs_sub_files_failed ON
                    wmbs_sub_files_failed.fileid = wmbs_fileset_files.fileid AND
                    wmbs_sub_files_failed.subscription = %d
                  INNER JOIN wmbs_file_runlumi_map ON
                    wmbs_file_runlumi_map.fileid = wmbs_fileset_files.fileid AND
                    wmbs_file_runlumi_map.run = :run
                  WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                  AND wmbs_sub_files_complete.fileid IS NULL
                  AND wmbs_sub_files_failed.fileid IS NULL
                  """ % (expressmergeSubscription['id'],
                         expressmergeSubscription['id'],
                         expressmergeSubscription['id'])

    bindVars = {"run": runNumber}
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    logging.debug("Unfinished mergepack jobs for run %d" % runNumber)

    if results[0][0] != 0:
        return False

    return True
