#!/usr/bin/env python
"""
_InsertCleanup_

Queries used for cleanup
"""

__version__ = "$Revision: 1.16 $"
__revision__ = "$Id: InsertCleanup.py,v 1.16 2010/07/07 08:44:35 hufnagel Exp $"

import logging
import time

from T0 import Globals

def updateDeletableExpressStreamers(dbConn):
    """
    _updateDeletableExpressStreamers_

    finds all deletable express streamers

      - have children (ie. express processing is done)
      - all children are express merged

    updates the deleted flag to 1

    """
    expressmergeSubscription = Globals.retrieveExpressMergeSubscription()

    sqlQuery = """MERGE INTO streamer a
                  USING (
                    SELECT streamer.streamer_id
                    FROM streamer
                    INNER JOIN repack_streamer_assoc ON
                      repack_streamer_assoc.streamer_id = streamer.streamer_id
                    INNER JOIN run_stream_style_assoc ON
                      run_stream_style_assoc.run_id = streamer.run_id AND
                      run_stream_style_assoc.stream_id = streamer.stream_id AND
                      run_stream_style_assoc.style_id = (SELECT id FROM processing_style WHERE name = 'Express')
                    LEFT OUTER JOIN wmbs_sub_files_complete ON
                      wmbs_sub_files_complete.fileid = repack_streamer_assoc.repacked_id AND
                      wmbs_sub_files_complete.subscription = %d
                    WHERE checkForZeroOneState(streamer.deleted) = 0
                    GROUP BY streamer.streamer_id
                    HAVING COUNT(*) = COUNT(wmbs_sub_files_complete.fileid)
                  ) b ON (b.streamer_id = a.streamer_id)
                  WHEN MATCHED THEN UPDATE
                  SET a.deleted = 1
                  """ % (expressmergeSubscription['id'])

    dbConn.execute(sqlQuery, useTransaction = False)

    return

def updateDeletableRepackStreamers(dbConn):
    """
    _updateDeletableRepackStreamers_

    finds all deletable repack streamers

      - have children (ie. repack processing is done)
      - all unmerged children are merged
      - all final RAW from streamer has been transfered
      - final RAW from streamer that needs to be recoed
        has the reco jobs finished and suceeded

    updates the deleted flag to 1

    """
    mergeSubscription = Globals.retrieveMergeSubscription()
    transferSubscription = Globals.retrieveTransferSubscription()
    recoSubscription = Globals.retrieveRecoSubscription()

    sqlQuery = """MERGE INTO streamer a
                  USING (
                    SELECT final_raw_files.streamer_id FROM (
                      SELECT streamer.streamer_id AS streamer_id,
                             CASE
                               WHEN unmerged_files.fileid IS NULL THEN repack_streamer_assoc.repacked_id
                               ELSE wmbs_file_parent.child
                             END AS fileid
                      FROM streamer
                      INNER JOIN repack_streamer_assoc  ON
                        repack_streamer_assoc.streamer_id = streamer.streamer_id
                      INNER JOIN run_stream_style_assoc ON
                        run_stream_style_assoc.run_id = streamer.run_id AND
                        run_stream_style_assoc.stream_id = streamer.stream_id AND
                        run_stream_style_assoc.style_id = (SELECT id FROM processing_style WHERE name = 'Bulk')
                      LEFT OUTER JOIN (
                        SELECT wmbs_fileset_files.fileid AS fileid
                        FROM wmbs_fileset_files
                        WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                      ) unmerged_files ON
                        unmerged_files.fileid = repack_streamer_assoc.repacked_id
                      LEFT OUTER JOIN wmbs_file_parent ON
                        wmbs_file_parent.parent = unmerged_files.fileid
                      WHERE checkForZeroOneState(streamer.deleted) = 0
                    ) final_raw_files
                    LEFT OUTER JOIN (
                      SELECT fileid AS fileid
                      FROM wmbs_sub_files_complete
                      WHERE subscription = %d
                      UNION
                      SELECT fileid AS fileid
                      FROM wmbs_sub_files_failed
                      WHERE subscription = %d
                    ) transfered_files ON
                      transfered_files.fileid = final_raw_files.fileid
                    LEFT OUTER JOIN (
                      SELECT fileid AS fileid
                      FROM wmbs_sub_files_complete
                      WHERE subscription = %d
                    ) reconstructed_files ON
                      reconstructed_files.fileid = final_raw_files.fileid
                    GROUP by final_raw_files.streamer_id
                    HAVING COUNT(*) = COUNT(final_raw_files.fileid)
                    AND COUNT(*) = COUNT(transfered_files.fileid)
                    AND COUNT(*) = COUNT(reconstructed_files.fileid)
                  ) b ON (b.streamer_id = a.streamer_id)
                  WHEN MATCHED THEN UPDATE
                  SET a.deleted = 1
                  """ % (mergeSubscription['id'],
                         transferSubscription['id'],
                         transferSubscription['id'],
                         recoSubscription['id'])

    dbConn.execute(sqlQuery, useTransaction = False)

    return

def updateDeletedStreamers(dbConn, streamerIDs):
    """
    _updateDeletedStreamers_

    updated deleted flag to 2

    """
    sqlQuery = """UPDATE streamer
                  SET deleted = 2
                  WHERE streamer_id = :STREAMER_ID
                  """

    bindVarList = []
    for streamerID in streamerIDs:
        bindVarList.append( { "STREAMER_ID" : streamerID } )

    if len(bindVarList) > 0:
        dbConn.execute(sqlQuery, bindVarList, useTransaction = False)

    return

def updateDeletableFiles(dbConn):
    """
    _updateDeletableFiles_

    finds all deletable files

      - files are successfully processed in all their subscriptions
      - exclude wmbsPublishSubscription from that check
      - files already in delete or expressDBS fileset are excluded

    inserts files into delete fileset

    """
    deleteSubscription = Globals.retrieveDeleteSubscription()
    wmbsPublishSubscription = Globals.retrieveWMBSPublishSubscription()
    expressDBSSubscription = Globals.retrieveExpressDBSSubscription()
    recoSubscription = Globals.retrieveRecoSubscription()
    alcaSkimSubscription = Globals.retrieveAlcaSubscription()
    transferSubscription = Globals.retrieveTransferSubscription()

    sqlQuery = """INSERT INTO wmbs_fileset_files a
                  (FILEID, FILESET, INSERT_TIME)
                  SELECT b.fileid, %d, :insert_time
                  FROM wmbs_fileset_files b
                  INNER JOIN wmbs_subscription c ON
                    c.fileset = b.fileset AND
                    c.id NOT IN ( %d )
                  LEFT OUTER JOIN wmbs_subscription d ON
                    d.id = c.id AND
                    d.id NOT IN ( %d, %d, %d )
                  LEFT OUTER JOIN wmbs_sub_files_complete ON
                    wmbs_sub_files_complete.fileid = b.fileid AND
                    wmbs_sub_files_complete.subscription = d.id
                  GROUP BY b.fileid
                  HAVING COUNT(*) = COUNT(d.id) AND
                  COUNT(*) = COUNT(wmbs_sub_files_complete.fileid)
                  """ % (deleteSubscription['id'],
                         wmbsPublishSubscription['id'],
                         wmbsPublishSubscription['id'],
                         expressDBSSubscription['id'],
                         deleteSubscription['id'])

    bindVars = { 'insert_time' : int(time.time()) }
    dbConn.execute(sqlQuery, bindVars, useTransaction = False)

    return

def updateDeletedFiles(dbConn, fileIDs):
    """
    _updateDeletedFiles_

    marks files as complete against the delete subscription

    """
    deleteSubscription = Globals.retrieveDeleteSubscription()

    sqlQuery = """INSERT INTO wmbs_sub_files_complete
                  (SUBSCRIPTION, FILEID)
                  VALUES (%d, :fileid)
                  """ % deleteSubscription['id']

    bindVarList = []
    for fileID in fileIDs:
        bindVarList.append( { "fileid" : fileID } )

    if len(bindVarList) > 0:
        dbConn.execute(sqlQuery, bindVarList, useTransaction = False)

    return
